@@ -90,7 +90,7 @@ static int __get_drm_device_name(int fd, char *name)
 	return -1;
 }
 
-static bool is_i915_device(int fd)
+bool is_i915_device(int fd)
 {
 	int ret;
 	char name[5] = "";
@@ -110,7 +110,7 @@ static bool is_vc4_device(int fd)
 	return !ret && strcmp("vc4", name) == 0;
 }
 
-static bool is_intel(int fd)
+static bool has_known_intel_chipset(int fd)
 {
 	struct drm_i915_getparam gp;
 	int devid = 0;
@@ -129,18 +129,6 @@ static bool is_intel(int fd)
 	return true;
 }
 
-static void check_stop_rings(void)
-{
-	enum stop_ring_flags flags;
-	flags = igt_get_stop_rings();
-	igt_warn_on_f(flags != 0,
-		      "i915_ring_stop flags on exit 0x%x, can't quiescent gpu cleanly\n",
-		      flags);
-
-	if (flags)
-		igt_set_stop_rings(STOP_RING_NONE);
-}
-
 #define LOCAL_I915_EXEC_VEBOX	(4 << 0)
 /**
  * gem_quiescent_gpu:
@@ -156,41 +144,33 @@ static void check_stop_rings(void)
  */
 void gem_quiescent_gpu(int fd)
 {
-	uint32_t batch[2] = {MI_BATCH_BUFFER_END, 0};
+	uint32_t bbe = MI_BATCH_BUFFER_END;
 	struct drm_i915_gem_execbuffer2 execbuf;
-	struct drm_i915_gem_exec_object2 gem_exec[1];
+	struct drm_i915_gem_exec_object2 obj;
+	unsigned ring;
 
-	check_stop_rings();
-
-	memset(gem_exec, 0, sizeof(gem_exec));
-	gem_exec[0].handle = gem_create(fd, 4096);
-	gem_write(fd, gem_exec[0].handle, 0, batch, sizeof(batch));
+	memset(&obj, 0, sizeof(obj));
+	obj.handle = gem_create(fd, 4096);
+	gem_write(fd, obj.handle, 0, &bbe, sizeof(&bbe));
 
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)gem_exec;
+	execbuf.buffers_ptr = (uintptr_t)&obj;
 	execbuf.buffer_count = 1;
 
-	execbuf.flags = I915_EXEC_RENDER;
-	gem_execbuf(fd, &execbuf);
-
-	if (gem_has_blt(fd)) {
-		execbuf.flags = I915_EXEC_BLT;
-		gem_execbuf(fd, &execbuf);
+	for (ring = 0; ring < 1<<6; ring++) {
+		execbuf.flags = ring;
+		__gem_execbuf(fd, &execbuf);
 	}
 
-	if (gem_has_bsd(fd)) {
-		execbuf.flags = I915_EXEC_BSD;
-		gem_execbuf(fd, &execbuf);
+	if (gem_has_bsd2(fd)) {
+		execbuf.flags = I915_EXEC_BSD | (2 << 13);
+		__gem_execbuf(fd, &execbuf);
 	}
 
-	if (gem_has_vebox(fd)) {
-		execbuf.flags = LOCAL_I915_EXEC_VEBOX;
-		gem_execbuf(fd, &execbuf);
-	}
+	gem_sync(fd, obj.handle);
+	gem_close(fd, obj.handle);
 
-	gem_sync(fd, gem_exec[0].handle);
 	igt_drop_caches_set(DROP_RETIRE);
-	gem_close(fd, gem_exec[0].handle);
 }
 
 /**
@@ -219,7 +199,7 @@ int drm_get_card(void)
 		if (fd == -1)
 			continue;
 
-		if (!is_i915_device(fd) || !is_intel(fd)) {
+		if (!is_i915_device(fd) || !has_known_intel_chipset(fd)) {
 			close(fd);
 			continue;
 		}
@@ -254,7 +234,9 @@ int __drm_open_driver(int chipset)
 		if (fd == -1)
 			continue;
 
-		found_intel =  is_i915_device(fd) && is_intel(fd) && (chipset & DRIVER_INTEL);
+		found_intel = is_i915_device(fd) &&
+			      has_known_intel_chipset(fd) &&
+			      (chipset & DRIVER_INTEL);
 
 		found_vc4 = is_vc4_device(fd) && (chipset & DRIVER_VC4);
 
@@ -285,7 +267,7 @@ static int __drm_open_driver_render(int chipset)
 		if (fd == -1)
 			continue;
 
-		if (!is_i915_device(fd) || !is_intel(fd)) {
+		if (!is_i915_device(fd) || !has_known_intel_chipset(fd)) {
 			close(fd);
 			fd = -1;
 			continue;
@@ -305,7 +287,6 @@ static void quiescent_gpu_at_exit(int sig)
 	if (at_exit_drm_fd < 0)
 		return;
 
-	check_stop_rings();
 	gem_quiescent_gpu(at_exit_drm_fd);
 	close(at_exit_drm_fd);
 	at_exit_drm_fd = -1;
@@ -316,7 +297,6 @@ static void quiescent_gpu_at_exit_render(int sig)
 	if (at_exit_drm_render_fd < 0)
 		return;
 
-	check_stop_rings();
 	gem_quiescent_gpu(at_exit_drm_render_fd);
 	close(at_exit_drm_render_fd);
 	at_exit_drm_render_fd = -1;
@@ -341,7 +321,7 @@ int drm_open_driver(int chipset)
 	if (__sync_fetch_and_add(&open_count, 1))
 		return fd;
 
-   if(chipset & DRIVER_INTEL){
+	if(chipset & DRIVER_INTEL){
 		gem_quiescent_gpu(fd);
 		igt_install_exit_handler(quiescent_gpu_at_exit);
 	}
@@ -399,3 +379,8 @@ int drm_open_driver_render(int chipset)
 
 	return fd;
 }
+
+void igt_require_intel(int fd)
+{
+	igt_require(is_i915_device(fd) && has_known_intel_chipset(fd));
+}
