@@ -82,6 +82,10 @@ int __drm_open_driver(int chipset);
 
 void gem_quiescent_gpu(int fd);
 
+void igt_require_intel(int fd);
+
+bool is_i915_device(int fd);
+
 /**
  * do_or_die:
  * @x: command
@@ -102,7 +106,7 @@ void gem_quiescent_gpu(int fd);
  * successfully executed.
  */
 #define do_ioctl(fd, ioc, ioc_data) do { \
-	igt_assert_eq(drmIoctl((fd), (ioc), (ioc_data)), 0); \
+	igt_assert_eq(igt_ioctl((fd), (ioc), (ioc_data)), 0); \
 	errno = 0; \
 } while (0)
 
@@ -117,7 +121,7 @@ void gem_quiescent_gpu(int fd);
  * returning a particular value in errno.
  */
 #define do_ioctl_err(fd, ioc, ioc_data, err) do { \
-	igt_assert_eq(drmIoctl((fd), (ioc), (ioc_data)), -1); \
+	igt_assert_eq(igt_ioctl((fd), (ioc), (ioc_data)), -1); \
 	igt_assert_eq(errno, err); \
 	errno = 0; \
 } while (0)
