@@ -40,7 +40,9 @@
 #include <signal.h>
 #include <pciaccess.h>
 #include <stdlib.h>
+#include <time.h>
 #include <unistd.h>
+#include <sys/poll.h>
 #include <sys/wait.h>
 #include <sys/time.h>
 #include <sys/types.h>
@@ -73,6 +75,205 @@
 
 
 /* signal interrupt helpers */
+
+#define MSEC_PER_SEC (1000)
+#define USEC_PER_SEC (1000*MSEC_PER_SEC)
+#define NSEC_PER_SEC (1000*USEC_PER_SEC)
+
+/* signal interrupt helpers */
+#define gettid() syscall(__NR_gettid)
+#define sigev_notify_thread_id _sigev_un._tid
+
+static struct __igt_sigiter_global {
+	pid_t tid;
+	timer_t timer;
+	struct timespec offset;
+	struct {
+		long hit, miss;
+		long ioctls, signals;
+	} stat;
+} __igt_sigiter;
+
+static void sigiter(int sig, siginfo_t *info, void *arg)
+{
+	__igt_sigiter.stat.signals++;
+}
+
+#if 0
+#define SIG_ASSERT(expr) igt_assert(expr)
+#else
+#define SIG_ASSERT(expr)
+#endif
+
+static int
+sig_ioctl(int fd, unsigned long request, void *arg)
+{
+	struct itimerspec its;
+	int ret;
+
+	SIG_ASSERT(__igt_sigiter.timer);
+	SIG_ASSERT(__igt_sigiter.tid == gettid());
+
+	memset(&its, 0, sizeof(its));
+	if (timer_settime(__igt_sigiter.timer, 0, &its, NULL)) {
+		/* oops, we didn't undo the interrupter (i.e. !unwound abort) */
+		igt_ioctl = drmIoctl;
+		return drmIoctl(fd, request, arg);
+	}
+
+	its.it_value = __igt_sigiter.offset;
+	do {
+		long serial;
+
+		__igt_sigiter.stat.ioctls++;
+
+		ret = 0;
+		serial = __igt_sigiter.stat.signals;
+		igt_assert(timer_settime(__igt_sigiter.timer, 0, &its, NULL) == 0);
+		if (ioctl(fd, request, arg))
+			ret = errno;
+		if (__igt_sigiter.stat.signals == serial)
+			__igt_sigiter.stat.miss++;
+		if (ret == 0)
+			break;
+
+		if (ret == EINTR) {
+			__igt_sigiter.stat.hit++;
+
+			its.it_value.tv_sec *= 2;
+			its.it_value.tv_nsec *= 2;
+			while (its.it_value.tv_nsec >= NSEC_PER_SEC) {
+				its.it_value.tv_nsec -= NSEC_PER_SEC;
+				its.it_value.tv_sec += 1;
+			}
+
+			SIG_ASSERT(its.it_value.tv_nsec >= 0);
+			SIG_ASSERT(its.it_value.tv_sec >= 0);
+		}
+	} while (ret == EAGAIN || ret == EINTR);
+
+	memset(&its, 0, sizeof(its));
+	timer_settime(__igt_sigiter.timer, 0, &its, NULL);
+
+	errno = ret;
+	return ret ? -1 : 0;
+}
+
+static bool igt_sigiter_start(struct __igt_sigiter *iter, bool enable)
+{
+	/* Note that until we can automatically clean up on failed/skipped
+	 * tests, we cannot assume the state of the igt_ioctl indirection.
+	 */
+	SIG_ASSERT(igt_ioctl == drmIoctl);
+	igt_ioctl = drmIoctl;
+
+	if (enable) {
+		struct timespec start, end;
+		struct sigevent sev;
+		struct sigaction act;
+		struct itimerspec its;
+
+		igt_ioctl = sig_ioctl;
+		__igt_sigiter.tid = gettid();
+
+		memset(&sev, 0, sizeof(sev));
+		sev.sigev_notify = SIGEV_SIGNAL | SIGEV_THREAD_ID;
+		sev.sigev_notify_thread_id = __igt_sigiter.tid;
+		sev.sigev_signo = SIGRTMIN;
+		igt_assert(timer_create(CLOCK_MONOTONIC, &sev, &__igt_sigiter.timer) == 0);
+
+		memset(&its, 0, sizeof(its));
+		igt_assert(timer_settime(__igt_sigiter.timer, 0, &its, NULL) == 0);
+
+		memset(&act, 0, sizeof(act));
+		act.sa_sigaction = sigiter;
+		act.sa_flags = SA_SIGINFO;
+		igt_assert(sigaction(SIGRTMIN, &act, NULL) == 0);
+
+		/* Try to find the approximate delay required to skip over
+		 * the timer_setttime and into the following ioctl() to try
+		 * and avoid the timer firing before we enter the drmIoctl.
+		 */
+		igt_assert(clock_gettime(CLOCK_MONOTONIC, &start) == 0);
+		igt_assert(timer_settime(__igt_sigiter.timer, 0, &its, NULL) == 0);
+		igt_assert(clock_gettime(CLOCK_MONOTONIC, &end) == 0);
+
+		__igt_sigiter.offset.tv_sec = end.tv_sec - start.tv_sec;
+		__igt_sigiter.offset.tv_nsec = end.tv_nsec - start.tv_nsec;
+		if (__igt_sigiter.offset.tv_nsec < 0) {
+			__igt_sigiter.offset.tv_nsec += NSEC_PER_SEC;
+			__igt_sigiter.offset.tv_sec -= 1;
+		}
+		if (__igt_sigiter.offset.tv_sec < 0) {
+			__igt_sigiter.offset.tv_nsec = 0;
+			__igt_sigiter.offset.tv_sec = 0;
+		}
+		igt_assert(__igt_sigiter.offset.tv_sec == 0);
+
+		igt_debug("Initial delay for interruption: %ld.%09lds\n",
+			  __igt_sigiter.offset.tv_sec,
+			  __igt_sigiter.offset.tv_nsec);
+	}
+
+	return true;
+}
+
+static bool igt_sigiter_stop(struct __igt_sigiter *iter, bool enable)
+{
+	if (enable) {
+		struct sigaction act;
+
+		SIG_ASSERT(igt_ioctl == sig_ioctl);
+		SIG_ASSERT(__igt_sigiter.tid == gettid());
+		igt_ioctl = drmIoctl;
+
+		timer_delete(__igt_sigiter.timer);
+
+		memset(&act, 0, sizeof(act));
+		act.sa_handler = SIG_IGN;
+		sigaction(SIGRTMIN, &act, NULL);
+
+		memset(&__igt_sigiter, 0, sizeof(__igt_sigiter));
+	}
+
+	memset(iter, 0, sizeof(*iter));
+	return false;
+}
+
+bool __igt_sigiter_continue(struct __igt_sigiter *iter, bool enable)
+{
+	if (iter->pass++ == 0)
+		return igt_sigiter_start(iter, enable);
+
+	/* If nothing reported SIGINT, nothing will on the next pass, so
+	 * give up! Also give up if everything is now executing faster
+	 * than current sigtimer.
+	 */
+	if (__igt_sigiter.stat.hit == 0 ||
+	    __igt_sigiter.stat.miss == __igt_sigiter.stat.ioctls)
+		return igt_sigiter_stop(iter, enable);
+
+	igt_debug("%s: pass %d, missed %ld/%ld\n",
+		  __func__, iter->pass - 1,
+		  __igt_sigiter.stat.miss,
+		  __igt_sigiter.stat.ioctls);
+
+	SIG_ASSERT(igt_ioctl == sig_ioctl);
+	SIG_ASSERT(__igt_sigiter.timer);
+
+	__igt_sigiter.offset.tv_sec *= 2;
+	__igt_sigiter.offset.tv_nsec *= 2;
+	while (__igt_sigiter.offset.tv_nsec >= NSEC_PER_SEC) {
+		__igt_sigiter.offset.tv_nsec -= NSEC_PER_SEC;
+		__igt_sigiter.offset.tv_sec += 1;
+	}
+	SIG_ASSERT(__igt_sigiter.offset.tv_nsec >= 0);
+	SIG_ASSERT(__igt_sigiter.offset.tv_sec >= 0);
+
+	memset(&__igt_sigiter.stat, 0, sizeof(__igt_sigiter.stat));
+	return true;
+}
+
 static struct igt_helper_process signal_helper;
 long long int sig_stat;
 static void __attribute__((noreturn)) signal_helper_process(pid_t pid)
@@ -103,6 +304,10 @@ static void sig_handler(int i)
  *
  * In tests with subtests this function can be called outside of failure
  * catching code blocks like #igt_fixture or #igt_subtest.
+ *
+ * Note that this just spews signals at the current process unconditionally and
+ * hence incurs quite a bit of overhead. For a more focused approach, with less
+ * overhead, look at the #igt_while_interruptible code block macro.
  */
 void igt_fork_signal_helper(void)
 {
@@ -147,6 +352,86 @@ void igt_stop_signal_helper(void)
 	sig_stat = 0;
 }
 
+#if HAVE_UDEV
+#include <libudev.h>
+
+static struct igt_helper_process hang_detector;
+static void __attribute__((noreturn))
+hang_detector_process(pid_t pid, dev_t rdev)
+{
+	struct udev_monitor *mon =
+		udev_monitor_new_from_netlink(udev_new(), "kernel");
+	struct pollfd pfd;
+
+	udev_monitor_filter_add_match_subsystem_devtype(mon, "drm", NULL);
+	udev_monitor_enable_receiving(mon);
+
+	pfd.fd = udev_monitor_get_fd(mon);
+	pfd.events = POLLIN;
+
+	while (poll(&pfd, 1, -1) > 0) {
+		struct udev_device *dev = udev_monitor_receive_device(mon);
+		dev_t devnum;
+
+		if (dev == NULL)
+			continue;
+
+		devnum = udev_device_get_devnum(dev);
+		if (memcmp(&rdev, &devnum, sizeof(dev_t)) == 0) {
+			const char *str;
+
+			str = udev_device_get_property_value(dev, "ERROR");
+			if (str && atoi(str) == 1)
+				kill(pid, SIGRTMAX);
+		}
+
+		udev_device_unref(dev);
+		if (kill(pid, 0)) /* Parent has died, so must we. */
+			break;
+	}
+
+	exit(0);
+}
+
+static void sig_abort(int sig)
+{
+	errno = 0; /* inside a signal, last errno reporting is confusing */
+	igt_assert(!"GPU hung");
+}
+
+void igt_fork_hang_detector(int fd)
+{
+	struct stat st;
+
+	if (igt_only_list_subtests())
+		return;
+
+	igt_assert(fstat(fd, &st) == 0);
+
+	signal(SIGRTMAX, sig_abort);
+	igt_fork_helper(&hang_detector)
+		hang_detector_process(getppid(), st.st_rdev);
+}
+
+void igt_stop_hang_detector(void)
+{
+	if (igt_only_list_subtests())
+		return;
+
+	igt_stop_helper(&hang_detector);
+}
+#else
+void igt_fork_hang_detector(int fd)
+{
+	if (igt_only_list_subtests())
+		return;
+}
+
+void igt_stop_hang_detector(void)
+{
+}
+#endif
+
 /**
  * igt_check_boolean_env_var:
  * @env_var: environment variable name
@@ -354,6 +639,8 @@ void igt_cleanup_aperture_trashers(void)
 	free(trash_bos);
 }
 
+#define SQUELCH ">/dev/null 2>&1"
+
 /**
  * igt_system_suspend_autoresume:
  *
@@ -364,18 +651,15 @@ void igt_cleanup_aperture_trashers(void)
  */
 void igt_system_suspend_autoresume(void)
 {
-	int ret;
-
 	/* FIXME: Simulation doesn't like suspend/resume, and not even a lighter
 	 * approach using /sys/power/pm_test to just test our driver's callbacks
 	 * seems to fare better. We need to investigate what's going on. */
 	igt_skip_on_simulation();
 
 	/* skip if system doesn't support suspend-to-mem */
-	igt_skip_on(system("rtcwake -n -s 30 -m mem") != 0);
+	igt_require(system("rtcwake -n -s 15 -m mem" SQUELCH) == 0);
 
-	ret = system("rtcwake -s 30 -m mem");
-	igt_assert_f(ret == 0,
+	igt_assert_f(system("rtcwake -s 15 -m mem") == 0,
 		     "This failure means that something is wrong with the "
 		     "rtcwake tool or how your distro is set up. This is not "
 		     "a i915.ko or i-g-t bug.\n");
@@ -391,8 +675,6 @@ void igt_system_suspend_autoresume(void)
  */
 void igt_system_hibernate_autoresume(void)
 {
-	int ret;
-
 	/* FIXME: I'm guessing simulation behaves the same way as with
 	 * suspend/resume, but it might be prudent to make sure
 	 */
@@ -402,13 +684,12 @@ void igt_system_hibernate_autoresume(void)
 	igt_skip_on_simulation();
 
 	/* skip if system doesn't support suspend-to-disk */
-	igt_skip_on(system("rtcwake -n -s 90 -m disk") != 0);
+	igt_require(system("rtcwake -n -s 30 -m disk" SQUELCH) == 0);
 
 	/* The timeout might need to be adjusted if hibernation takes too long
 	 * or if we have to wait excessively long before resume
 	 */
-	ret = system("rtcwake -s 90 -m disk");
-	igt_assert_f(ret == 0,
+	igt_assert_f(system("rtcwake -s 30 -m disk") == 0,
 		     "This failure means that something is wrong with the "
 		     "rtcwake tool or how your distro is set up. This is not "
 		     "a i915.ko or i-g-t bug.\n");
@@ -639,35 +920,58 @@ struct type_name {
 	const char *name;
 };
 
-#define type_name_fn(res) \
-const char * kmstest_##res##_str(int type) {		\
-	unsigned int i;					\
-	for (i = 0; i < ARRAY_SIZE(res##_names); i++) { \
-		if (res##_names[i].type == type)	\
-			return res##_names[i].name;	\
-	}						\
-	return "(invalid)";				\
+static const char *find_type_name(const struct type_name *names, int type)
+{
+	for (; names->name; names++) {
+		if (names->type == type)
+			return names->name;
+	}
+
+	return "(invalid)";
 }
 
-struct type_name encoder_type_names[] = {
+static const struct type_name encoder_type_names[] = {
 	{ DRM_MODE_ENCODER_NONE, "none" },
 	{ DRM_MODE_ENCODER_DAC, "DAC" },
 	{ DRM_MODE_ENCODER_TMDS, "TMDS" },
 	{ DRM_MODE_ENCODER_LVDS, "LVDS" },
 	{ DRM_MODE_ENCODER_TVDAC, "TVDAC" },
+	{ DRM_MODE_ENCODER_VIRTUAL, "Virtual" },
+	{ DRM_MODE_ENCODER_DSI, "DSI" },
+	{ DRM_MODE_ENCODER_DPMST, "DP MST" },
+	{}
 };
 
-type_name_fn(encoder_type)
+/**
+ * kmstest_encoder_type_str:
+ * @type: DRM_MODE_ENCODER_* enumeration value
+ *
+ * Returns: A string representing the drm encoder @type.
+ */
+const char *kmstest_encoder_type_str(int type)
+{
+	return find_type_name(encoder_type_names, type);
+}
 
-struct type_name connector_status_names[] = {
+static const struct type_name connector_status_names[] = {
 	{ DRM_MODE_CONNECTED, "connected" },
 	{ DRM_MODE_DISCONNECTED, "disconnected" },
 	{ DRM_MODE_UNKNOWNCONNECTION, "unknown" },
+	{}
 };
 
-type_name_fn(connector_status)
+/**
+ * kmstest_connector_status_str:
+ * @status: DRM_MODE_* connector status value
+ *
+ * Returns: A string representing the drm connector status @status.
+ */
+const char *kmstest_connector_status_str(int status)
+{
+	return find_type_name(connector_status_names, status);
+}
 
-struct type_name connector_type_names[] = {
+static const struct type_name connector_type_names[] = {
 	{ DRM_MODE_CONNECTOR_Unknown, "unknown" },
 	{ DRM_MODE_CONNECTOR_VGA, "VGA" },
 	{ DRM_MODE_CONNECTOR_DVII, "DVI-I" },
@@ -683,10 +987,21 @@ struct type_name connector_type_names[] = {
 	{ DRM_MODE_CONNECTOR_HDMIB, "HDMI-B" },
 	{ DRM_MODE_CONNECTOR_TV, "TV" },
 	{ DRM_MODE_CONNECTOR_eDP, "eDP" },
+	{ DRM_MODE_CONNECTOR_VIRTUAL, "Virtual" },
+	{ DRM_MODE_CONNECTOR_DSI, "DSI" },
+	{}
 };
 
-type_name_fn(connector_type)
-
+/**
+ * kmstest_connector_type_str:
+ * @type: DRM_MODE_CONNECTOR_* enumeration value
+ *
+ * Returns: A string representing the drm connector @type.
+ */
+const char *kmstest_connector_type_str(int type)
+{
+	return find_type_name(connector_type_names, type);
+}
 
 /**
  * igt_lock_mem:
