@@ -97,7 +97,9 @@
  *
  * To allow this i-g-t provides #igt_fixture code blocks for setup code outside
  * of subtests and automatically skips the subtest code blocks themselves. For
- * special cases igt_only_list_subtests() is also provided.
+ * special cases igt_only_list_subtests() is also provided. For setup code only
+ * shared by a group of subtest encapsulate the #igt_fixture block and all the
+ * subtestest in a #igt_subtest_group block.
  *
  * # Magic Control Blocks
  *
@@ -396,6 +398,23 @@ error:
 	return -errno;
 }
 
+#define MSEC_PER_SEC (1000)
+#define USEC_PER_SEC (1000*MSEC_PER_SEC)
+#define NSEC_PER_SEC (1000*USEC_PER_SEC)
+uint64_t igt_nsec_elapsed(struct timespec *start)
+{
+	struct timespec now;
+
+	gettime(&now);
+	if ((start->tv_sec | start->tv_nsec) == 0) {
+		*start = now;
+		return 0;
+	}
+
+	return ((now.tv_nsec - start->tv_nsec) +
+		(uint64_t)NSEC_PER_SEC*(now.tv_sec - start->tv_sec));
+}
+
 bool __igt_fixture(void)
 {
 	assert(!in_fixture);
@@ -872,6 +891,16 @@ bool igt_only_list_subtests(void)
 	return list_subtests;
 }
 
+void __igt_subtest_group_save(int *save)
+{
+	*save = skip_subtests_henceforth;
+}
+
+void __igt_subtest_group_restore(int save)
+{
+	skip_subtests_henceforth = save;
+}
+
 static bool skipped_one = false;
 static bool succeeded_one = false;
 static bool failed_one = false;
@@ -1124,10 +1153,22 @@ void igt_exit(void)
 		exit(IGT_EXIT_INVALID);
 	}
 
-
 	if (igt_only_list_subtests())
 		exit(IGT_EXIT_SUCCESS);
 
+	/* Calling this without calling one of the above is a failure */
+	assert(!test_with_subtests ||
+	       skipped_one ||
+	       succeeded_one ||
+	       failed_one);
+
+	if (test_with_subtests && !failed_one) {
+		if (succeeded_one)
+			igt_exitcode = IGT_EXIT_SUCCESS;
+		else
+			igt_exitcode = IGT_EXIT_SKIP;
+	}
+
 	kmsg(KERN_INFO "%s: exiting, ret=%d\n", command_str, igt_exitcode);
 	igt_debug("Exiting with status code %d\n", igt_exitcode);
 
@@ -1154,19 +1195,11 @@ void igt_exit(void)
 				result = "FAIL";
 		}
 
-		printf("%s (%.3fs)\n", result, time_elapsed(&subtest_time, &now));
-		exit(igt_exitcode);
+		printf("%s (%.3fs)\n",
+		       result, time_elapsed(&subtest_time, &now));
 	}
 
-	/* Calling this without calling one of the above is a failure */
-	assert(skipped_one || succeeded_one || failed_one);
-
-	if (failed_one)
-		exit(igt_exitcode);
-	else if (succeeded_one)
-		exit(IGT_EXIT_SUCCESS);
-	else
-		exit(IGT_EXIT_SKIP);
+	exit(igt_exitcode);
 }
 
 /* fork support code */
@@ -1276,6 +1309,13 @@ int igt_wait_helper(struct igt_helper_process *proc)
 	return status;
 }
 
+static bool helper_was_alive(struct igt_helper_process *proc,
+			     int status)
+{
+	return (WIFSIGNALED(status) &&
+		WTERMSIG(status) == (proc->use_SIGKILL ? SIGKILL : SIGTERM));
+}
+
 /**
  * igt_stop_helper:
  * @proc: #igt_helper_process structure
@@ -1291,8 +1331,9 @@ void igt_stop_helper(struct igt_helper_process *proc)
 	kill(proc->pid, proc->use_SIGKILL ? SIGKILL : SIGTERM);
 
 	status = igt_wait_helper(proc);
-	assert(WIFSIGNALED(status) &&
-	       WTERMSIG(status) == (proc->use_SIGKILL ? SIGKILL : SIGTERM));
+	if (!helper_was_alive(proc, status))
+		igt_debug("Helper died too early with status=%d\n", status);
+	assert(helper_was_alive(proc, status));
 }
 
 static void children_exit_handler(int sig)
@@ -1406,6 +1447,8 @@ void igt_waitchildren(void)
 
 /**
  * igt_waitchildren_timeout:
+ * @seconds: timeout in seconds to wait
+ * @reason: debug string explaining what timedout
  *
  * Wait for all children forked with igt_fork, for a maximum of @seconds.
  *
@@ -1433,7 +1476,6 @@ static struct {
 
 static igt_exit_handler_t exit_handler_fn[MAX_EXIT_HANDLERS];
 static bool exit_handler_disabled;
-static sigset_t saved_sig_mask;
 #define SIGDEF(x) { x, #x, sizeof(#x) - 1 }
 #define SILENT(x) { x, NULL, 0 }
 static const struct { int number; const char *name; size_t name_len; } handled_signals[] =
@@ -1607,51 +1649,6 @@ err:
 	igt_assert_f(0, "failed to install the signal handler\n");
 }
 
-/**
- * igt_disable_exit_handler:
- *
- * Temporarily disable all exit handlers. Useful for library code doing tricky
- * things.
- */
-void igt_disable_exit_handler(void)
-{
-	sigset_t set;
-	int i;
-
-	if (exit_handler_disabled)
-		return;
-
-	sigemptyset(&set);
-	for (i = 0; i < ARRAY_SIZE(handled_signals); i++)
-		sigaddset(&set, handled_signals[i].number);
-
-	if (sigprocmask(SIG_BLOCK, &set, &saved_sig_mask)) {
-		perror("sigprocmask");
-		return;
-	}
-
-	exit_handler_disabled = true;
-}
-
-/**
- * igt_enable_exit_handler:
- *
- * Re-enable all exit handlers temporarily disabled with
- * igt_disable_exit_handler().
- */
-void igt_enable_exit_handler(void)
-{
-	if (!exit_handler_disabled)
-		return;
-
-	if (sigprocmask(SIG_SETMASK, &saved_sig_mask, NULL)) {
-		perror("sigprocmask");
-		return;
-	}
-
-	exit_handler_disabled = false;
-}
-
 /* simulation enviroment support */
 
 /**
@@ -1885,6 +1882,10 @@ FILE *__igt_fopen_data(const char* igt_srcdir, const char* igt_datadir,
 		snprintf(path, sizeof(path), "%s/%s", igt_srcdir, filename);
 		fp = fopen(path, "r");
 	}
+	if (!fp) {
+		snprintf(path, sizeof(path), "./%s", filename);
+		fp = fopen(path, "r");
+	}
 
 	if (!fp)
 		igt_critical("Could not open data file \"%s\": %s", filename,
