@@ -137,7 +137,7 @@ igt_hang_ring_t igt_hang_ctx(int fd,
 	struct drm_i915_gem_execbuffer2 execbuf;
 	struct drm_i915_gem_exec_object2 exec;
 	struct local_i915_gem_context_param param;
-	uint32_t b[8];
+	uint32_t b[16];
 	unsigned ban;
 	unsigned len;
 
@@ -181,21 +181,21 @@ igt_hang_ring_t igt_hang_ctx(int fd,
 	exec.relocs_ptr = (uintptr_t)&reloc;
 
 	memset(b, 0xc5, sizeof(b));
+	b[0] = 0xffffffff;
 	len = 2;
 	if (intel_gen(intel_get_drm_devid(fd)) >= 8)
 		len++;
-	b[0] = MI_BATCH_BUFFER_START | (len - 2);
-	b[len] = MI_BATCH_BUFFER_END;
-	b[len+1] = MI_NOOP;
+	b[1] = MI_BATCH_BUFFER_START | (len - 2);
+	b[1+len] = MI_BATCH_BUFFER_END;
+	b[2+len] = MI_NOOP;
 	gem_write(fd, exec.handle, 0, b, sizeof(b));
 
-	reloc.offset = 4;
+	reloc.offset = 8;
 	reloc.target_handle = exec.handle;
 	reloc.read_domains = I915_GEM_DOMAIN_COMMAND;
 
 	execbuf.buffers_ptr = (uintptr_t)&exec;
 	execbuf.buffer_count = 1;
-	execbuf.batch_len = sizeof(b);
 	execbuf.flags = ring;
 	i915_execbuffer2_set_context_id(execbuf, ctx);
 	gem_execbuf(fd, &execbuf);
@@ -225,10 +225,14 @@ igt_hang_ring_t igt_hang_ring(int fd, int ring)
 
 static void eat_error_state(void)
 {
-	int fd;
+	int fd, ret;
 
 	fd = igt_debugfs_open("i915_error_state", O_WRONLY);
-	igt_assert(write(fd, "", 1) == 1);
+	do {
+		ret = write(fd, "", 1);
+		if (ret < 0)
+			ret = -errno;
+	} while (ret == -EINTR || ret == -EAGAIN);
 	close(fd);
 }
 
@@ -364,115 +368,9 @@ int igt_open_forcewake_handle(void)
 		return -1;
 	return igt_debugfs_open("i915_forcewake_user", O_WRONLY);
 }
-
-/**
- * igt_to_stop_ring_flag:
- * @ring: the specified ring flag from execbuf ioctl (I915_EXEC_*)
- *
- * This converts the specified ring to a ring flag to be used
- * with igt_get_stop_rings() and igt_set_stop_rings().
- *
- * Returns:
- * Ring flag for the given ring.
- */
-enum stop_ring_flags igt_to_stop_ring_flag(int ring) {
-	if (ring == I915_EXEC_DEFAULT)
-		return STOP_RING_RENDER;
-
-	igt_assert(ring && ((ring & ~I915_EXEC_RING_MASK) == 0));
-	return 1 << (ring - 1);
-}
-
-static void stop_rings_write(uint32_t mask)
-{
-	int fd;
-	char buf[80];
-
-	igt_assert(snprintf(buf, sizeof(buf), "0x%08x", mask) == 10);
-	fd = igt_debugfs_open("i915_ring_stop", O_WRONLY);
-	igt_assert(fd >= 0);
-
-	igt_assert(write(fd, buf, strlen(buf)) == strlen(buf));
-	close(fd);
-}
-
-/**
- * igt_get_stop_rings:
- *
- * Read current ring flags from 'i915_ring_stop' debugfs entry.
- *
- * Returns:
- * Current ring flags.
- */
-enum stop_ring_flags igt_get_stop_rings(void)
-{
-	int fd;
-	char buf[80];
-	int l;
-	unsigned long long ring_mask;
-
-	fd = igt_debugfs_open("i915_ring_stop", O_RDONLY);
-	igt_assert(fd >= 0);
-	l = read(fd, buf, sizeof(buf)-1);
-	igt_assert(l > 0);
-	igt_assert(l < sizeof(buf));
-
-	buf[l] = '\0';
-
-	close(fd);
-
-	errno = 0;
-	ring_mask = strtoull(buf, NULL, 0);
-	igt_assert(errno == 0);
-	return ring_mask;
-}
-
-/**
- * igt_set_stop_rings:
- * @flags: Ring flags to write
- *
- * This writes @flags to 'i915_ring_stop' debugfs entry. Driver will
- * prevent the CPU from writing tail pointer for the ring that @flags
- * specify. Note that the ring is not stopped right away. Instead any
- * further command emissions won't be executed after the flag is set.
- *
- * This is the least invasive way to make the GPU stuck. Hence you must
- * set this after a batch submission with it's own invalid or endless
- * looping instructions. In this case it is merely for giving notification
- * for the driver that this was simulated hang, as the batch would have
- * caused hang in any case. On the other hand if you use a valid or noop
- * batch and want to hang the ring (GPU), you must set corresponding flag
- * before submitting the batch.
- *
- * Driver checks periodically if a ring is making any progress, and if
- * it is not, it will declare the ring to be hung and will reset the GPU.
- * After reset, the driver will clear flags in 'i915_ring_stop'
- *
- * Note: Always when hanging the GPU, use igt_set_stop_rings() to
- * notify the driver. Driver controls hang log messaging based on
- * these flags and thus prevents false positives on logs.
- */
-void igt_set_stop_rings(enum stop_ring_flags flags)
-{
-	enum stop_ring_flags current;
-
-	igt_assert((flags & ~(STOP_RING_ALL |
-			      STOP_RING_ALLOW_BAN |
-			      STOP_RING_ALLOW_ERRORS)) == 0);
-
-	current = igt_get_stop_rings();
-	igt_assert_f(flags == 0 || current == 0,
-		     "previous i915_ring_stop is still 0x%x\n", current);
-
-	stop_rings_write(flags);
-	current = igt_get_stop_rings();
-	igt_warn_on_f(current != flags,
-		      "i915_ring_stop readback mismatch 0x%x vs 0x%x\n",
-		      flags, current);
-}
-
 static unsigned int clflush_size;
 
+#if defined(__x86_64__) || defined(__i386__)
 int igt_setup_clflush(void)
 {
 	FILE *file;
@@ -512,22 +410,32 @@ int igt_setup_clflush(void)
 	return has_clflush && clflush_size;
 }
 
+__attribute__((target("sse2")))
 void igt_clflush_range(void *addr, int size)
 {
-#if defined(__i386__) || defined(__x86_64__)
 	char *p, *end;
 
 	end = (char *)addr + size;
 	p = (char *)((uintptr_t)addr & ~((uintptr_t)clflush_size - 1));
 
-	asm volatile("mfence" ::: "memory");
+	__builtin_ia32_mfence();
 	for (; p < end; p += clflush_size)
-		asm volatile("clflush %0" : "+m" (*(volatile char *)p));
-	asm volatile("mfence" ::: "memory");
-#else
-	fprintf(stderr, "igt_clflush_range() unsupported\n");
-#endif
+		__builtin_ia32_clflush(p);
+	__builtin_ia32_clflush(end - 1); /* magic serialisation for byt+ */
+	__builtin_ia32_mfence();
 }
+#else
+int igt_setup_clflush(void)
+{
+	/* requires mfence + clflush, both SSE2 instructions */
+	return 0;
+}
+
+void igt_clflush_range(void *addr, int size)
+{
+	fprintf(stderr, "igt_clflush_range() unsupported\n");
+}
+#endif
 
 /**
  * intel_detect_and_clear_missed_irq:
