@@ -236,6 +236,8 @@ void intel_check_pch(void);
 #define PCI_CHIP_BROXTON_0		0x0A84
 #define PCI_CHIP_BROXTON_1		0x1A84
 #define PCI_CHIP_BROXTON_2		0x5A84
+#define PCI_CHIP_BROXTON_3		0x1A85
+#define PCI_CHIP_BROXTON_4		0x5A85
 
 #endif /* __GTK_DOC_IGNORE__ */
 
@@ -460,7 +462,9 @@ void intel_check_pch(void);
 
 #define IS_BROXTON(devid)	((devid) == PCI_CHIP_BROXTON_0 || \
 				 (devid) == PCI_CHIP_BROXTON_1 || \
-				 (devid) == PCI_CHIP_BROXTON_2)
+				 (devid) == PCI_CHIP_BROXTON_2 || \
+				 (devid) == PCI_CHIP_BROXTON_3 || \
+				 (devid) == PCI_CHIP_BROXTON_4)
 
 #define IS_GEN9(devid)		(IS_KABYLAKE(devid) || \
 				 IS_SKYLAKE(devid) || \
@@ -473,14 +477,6 @@ void intel_check_pch(void);
 				 IS_GEN8(devid) || \
 				 IS_GEN9(devid))
 
-#define IS_9XX(devid)		(IS_GEN3(devid) || \
-				 IS_GEN4(devid) || \
-				 IS_GEN5(devid) || \
-				 IS_GEN6(devid) || \
-				 IS_GEN7(devid) || \
-				 IS_GEN8(devid) || \
-				 IS_GEN9(devid))
-
 #define IS_INTEL(devid)		(IS_GEN2(devid) || \
 				 IS_GEN3(devid) || \
 				 IS_GEN4(devid) || \
