@@ -46,6 +46,7 @@
 #elif defined(HAVE_SWAPCTL) /* Solaris */
 #include <sys/swap.h>
 #endif
+#include <sys/resource.h>
 
 #include "intel_io.h"
 #include "drmtest.h"
@@ -64,11 +65,8 @@ intel_get_total_ram_mb(void)
 
 #ifdef HAVE_STRUCT_SYSINFO_TOTALRAM /* Linux */
 	struct sysinfo sysinf;
-	int ret;
-
-	ret = sysinfo(&sysinf);
-	igt_assert(ret == 0);
 
+	igt_assert(sysinfo(&sysinf) == 0);
 	retval = sysinf.totalram;
 	retval *= sysinf.mem_unit;
 #elif defined(_SC_PAGESIZE) && defined(_SC_PHYS_PAGES) /* Solaris */
@@ -98,18 +96,10 @@ intel_get_avail_ram_mb(void)
 
 #ifdef HAVE_STRUCT_SYSINFO_TOTALRAM /* Linux */
 	struct sysinfo sysinf;
-	int fd, ret;
 
-	fd = open("/proc/sys/vm/drop_caches", O_RDWR);
-	if (fd != -1) {
-		ret = write(fd, "3\n", 2);
-		close(fd);
-		(void)ret;
-	}
-
-	ret = sysinfo(&sysinf);
-	igt_assert(ret == 0);
+	intel_purge_vm_caches();
 
+	igt_assert(sysinfo(&sysinf) == 0);
 	retval = sysinf.freeram;
 	retval *= sysinf.mem_unit;
 #elif defined(_SC_PAGESIZE) && defined(_SC_AVPHYS_PAGES) /* Solaris */
@@ -139,11 +129,8 @@ intel_get_total_swap_mb(void)
 
 #ifdef HAVE_STRUCT_SYSINFO_TOTALRAM /* Linux */
 	struct sysinfo sysinf;
-	int ret;
-
-	ret = sysinfo(&sysinf);
-	igt_assert(ret == 0);
 
+	igt_assert(sysinfo(&sysinf) == 0);
 	retval = sysinf.freeswap;
 	retval *= sysinf.mem_unit;
 #elif defined(HAVE_SWAPCTL) /* Solaris */
@@ -206,6 +193,29 @@ static uint64_t vfs_file_max(void)
 	return max;
 }
 
+static unsigned max_open_files(void)
+{
+	struct rlimit rlim;
+
+	if (getrlimit(RLIMIT_NOFILE, &rlim))
+		rlim.rlim_cur = 64 << 10;
+
+	return rlim.rlim_cur;
+}
+
+/**
+ * intel_require_files:
+ * @count: number of files that will be created
+ *
+ * Does the system support enough file descriptors for the test?
+ */
+void intel_require_files(uint64_t count)
+{
+	igt_require_f(count < max_open_files(),
+		      "Estimated that we need %'llu files, but the process maximum is only %'llu\n",
+		      (long long)count, (long long)max_open_files());
+}
+
 int __intel_check_memory(uint64_t count, uint64_t size, unsigned mode,
 			 uint64_t *out_required, uint64_t *out_total)
 {
@@ -268,7 +278,7 @@ void intel_require_memory(uint64_t count, uint64_t size, unsigned mode)
 {
 	uint64_t required, total;
 
-	igt_skip_on_f(!__intel_check_memory(count, size, mode,
+	igt_require_f(__intel_check_memory(count, size, mode,
 					    &required, &total),
 		      "Estimated that we need %'llu objects and %'llu MiB for the test, but only have %'llu MiB available (%s%s) and a maximum of %'llu objects\n",
 		      (long long)count,
@@ -284,13 +294,18 @@ void intel_require_memory(uint64_t count, uint64_t size, unsigned mode)
 void
 intel_purge_vm_caches(void)
 {
+	static int once;
 	int fd;
 
 	fd = open("/proc/sys/vm/drop_caches", O_RDWR);
 	if (fd < 0)
 		return;
 
-	igt_assert_eq(write(fd, "3\n", 2), 2);
+	if (!once) {
+		igt_assert_eq(write(fd, "4\n", 2), 2); /* Be quiet! */
+		once = 1;
+	}
+	igt_assert_eq(write(fd, "3\n", 2), 2); /* Drop page/slab caches */
 	close(fd);
 }
 
