@@ -152,10 +152,12 @@ struct rect {
 
 #define MAX_CONNECTORS 32
 #define MAX_PLANES 32
+#define MAX_ENCODERS 32
 struct {
 	int fd;
 	drmModeResPtr res;
 	drmModeConnectorPtr connectors[MAX_CONNECTORS];
+	drmModeEncoderPtr encoders[MAX_ENCODERS];
 	drmModePlaneResPtr plane_res;
 	drmModePlanePtr planes[MAX_PLANES];
 	uint64_t plane_types[MAX_PLANES];
@@ -357,32 +359,45 @@ static drmModeConnectorPtr get_connector(uint32_t id)
 	igt_assert(false);
 }
 
+static drmModeEncoderPtr get_encoder(uint32_t id)
+{
+	int i;
+
+	for (i = 0; i < drm.res->count_encoders; i++)
+		if (drm.res->encoders[i] == id)
+			return drm.encoders[i];
+
+	igt_assert(false);
+}
+
 static void print_mode_info(const char *screen, struct modeset_params *params)
 {
 	drmModeConnectorPtr c = get_connector(params->connector_id);
 
-	igt_info("%s screen: %s %s\n",
+	igt_info("%s screen: %s %s, crtc %d\n",
 		 screen,
 		 kmstest_connector_type_str(c->connector_type),
-		 params->mode->name);
+		 params->mode->name,
+		 params->crtc_id);
 }
 
 static void init_mode_params(struct modeset_params *params, uint32_t crtc_id,
-			     int crtc_index, uint32_t connector_id,
+			     drmModeConnectorPtr connector,
 			     drmModeModeInfoPtr mode)
 {
 	uint32_t plane_id = 0;
+	int crtc_idx = kmstest_get_crtc_idx(drm.res, crtc_id);
 	int i;
 
 	for (i = 0; i < drm.plane_res->count_planes && plane_id == 0; i++)
-		if ((drm.planes[i]->possible_crtcs & (1 << crtc_index)) &&
+		if ((drm.planes[i]->possible_crtcs & (1 << crtc_idx)) &&
 		    drm.plane_types[i] == DRM_PLANE_TYPE_OVERLAY)
 			plane_id = drm.planes[i]->plane_id;
 
 	igt_assert(plane_id);
 
 	params->crtc_id = crtc_id;
-	params->connector_id = connector_id;
+	params->connector_id = connector->connector_id;
 	params->mode = mode;
 	params->sprite_id = plane_id;
 
@@ -427,56 +442,90 @@ static bool connector_get_mode(drmModeConnectorPtr c, drmModeModeInfoPtr *mode)
 	return true;
 }
 
-static bool init_modeset_cached_params(void)
+static bool connector_supports_pipe_a(drmModeConnectorPtr connector)
 {
 	int i;
-	uint32_t prim_connector_id = 0, scnd_connector_id = 0;
+
+	for (i = 0; i < connector->count_encoders; i++)
+		if (get_encoder(connector->encoders[i])->possible_crtcs & 1)
+			return true;
+
+	return false;
+}
+
+static bool find_connector(bool edp_only, bool pipe_a, uint32_t forbidden_id,
+			   drmModeConnectorPtr *ret_connector,
+			   drmModeModeInfoPtr *ret_mode)
+{
+	drmModeConnectorPtr c = NULL;
+	drmModeModeInfoPtr mode = NULL;
+	int i;
+
+	for (i = 0; i < drm.res->count_connectors; i++) {
+		c = drm.connectors[i];
+
+		if (edp_only && c->connector_type != DRM_MODE_CONNECTOR_eDP)
+			continue;
+		if (pipe_a && !connector_supports_pipe_a(c))
+			continue;
+		if (c->connector_id == forbidden_id)
+			continue;
+		if (!connector_get_mode(c, &mode))
+			continue;
+
+		*ret_connector = c;
+		*ret_mode = mode;
+		return true;
+	}
+
+	return false;
+}
+
+static bool init_modeset_cached_params(void)
+{
+	drmModeConnectorPtr prim_connector = NULL, scnd_connector = NULL;
 	drmModeModeInfoPtr prim_mode = NULL, scnd_mode = NULL;
-	drmModeModeInfoPtr tmp_mode;
+	uint32_t prim_crtc_id, scnd_crtc_id;
 
-	/* First, try to find an eDP monitor since it's the only possible type
-	 * for PSR.  */
-	for (i = 0; i < drm.res->count_connectors; i++) {
-		if (drm.connectors[i]->connector_type != DRM_MODE_CONNECTOR_eDP)
-			continue;
+	/*
+	 * We have this problem where PSR is only present on eDP monitors and
+	 * FBC is only present on pipe A for some platforms. So we search first
+	 * for the ideal case of eDP supporting pipe A, and try the less optimal
+	 * configs later, sacrificing  one of the features.
+	 * TODO: refactor the code in a way that allows us to have different
+	 * sets of prim/scnd structs for different features.
+	 */
+	find_connector(true, true, 0, &prim_connector, &prim_mode);
+	if (!prim_connector)
+		find_connector(true, false, 0, &prim_connector, &prim_mode);
+	if (!prim_connector)
+		find_connector(false, true, 0, &prim_connector, &prim_mode);
+	if (!prim_connector)
+		find_connector(false, false, 0, &prim_connector, &prim_mode);
 
-		if (connector_get_mode(drm.connectors[i], &tmp_mode)) {
-			prim_connector_id = drm.res->connectors[i];
-			prim_mode = tmp_mode;
-		}
-	}
-	for (i = 0; i < drm.res->count_connectors; i++) {
-		/* Don't pick again what we just selected on the above loop. */
-		if (drm.res->connectors[i] == prim_connector_id)
-			continue;
-
-		if (connector_get_mode(drm.connectors[i], &tmp_mode)) {
-			if (!prim_connector_id) {
-				prim_connector_id = drm.res->connectors[i];
-				prim_mode = tmp_mode;
-			} else if (!scnd_connector_id) {
-				scnd_connector_id = drm.res->connectors[i];
-				scnd_mode = tmp_mode;
-				break;
-			}
-		}
-	}
-
-	if (!prim_connector_id)
+	if (!prim_connector)
 		return false;
 
-	init_mode_params(&prim_mode_params, drm.res->crtcs[0], 0,
-			 prim_connector_id, prim_mode);
+	find_connector(false, false, prim_connector->connector_id,
+		       &scnd_connector, &scnd_mode);
+
+	prim_crtc_id = kmstest_find_crtc_for_connector(drm.fd, drm.res,
+						       prim_connector, 0);
+	init_mode_params(&prim_mode_params, prim_crtc_id,
+			 prim_connector, prim_mode);
 	print_mode_info("Primary", &prim_mode_params);
 
-	if (!scnd_connector_id) {
+	if (!scnd_connector) {
 		scnd_mode_params.connector_id = 0;
 		return true;
 	}
 
 	igt_assert(drm.res->count_crtcs >= 2);
-	init_mode_params(&scnd_mode_params, drm.res->crtcs[1], 1,
-			 scnd_connector_id, scnd_mode);
+	scnd_crtc_id = kmstest_find_crtc_for_connector(drm.fd, drm.res,
+						      scnd_connector,
+			1 << kmstest_get_crtc_idx(drm.res, prim_crtc_id));
+	init_mode_params(&scnd_mode_params, scnd_crtc_id,
+			 scnd_connector, scnd_mode);
 	print_mode_info("Secondary", &scnd_mode_params);
 
 	return true;
@@ -1283,10 +1332,14 @@ static void setup_drm(void)
 
 	drm.res = drmModeGetResources(drm.fd);
 	igt_assert(drm.res->count_connectors <= MAX_CONNECTORS);
+	igt_assert(drm.res->count_encoders <= MAX_ENCODERS);
 
 	for (i = 0; i < drm.res->count_connectors; i++)
 		drm.connectors[i] = drmModeGetConnectorCurrent(drm.fd,
 						drm.res->connectors[i]);
+	for (i = 0; i < drm.res->count_encoders; i++)
+		drm.encoders[i] = drmModeGetEncoder(drm.fd,
+						    drm.res->encoders[i]);
 
 	rc = drmSetClientCap(drm.fd, DRM_CLIENT_CAP_UNIVERSAL_PLANES, 1);
 	igt_require(rc == 0);
@@ -1314,6 +1367,8 @@ static void teardown_drm(void)
 		drmModeFreePlane(drm.planes[i]);
 	drmModeFreePlaneResources(drm.plane_res);
 
+	for (i = 0; i < drm.res->count_encoders; i++)
+		drmModeFreeEncoder(drm.encoders[i]);
 	for (i = 0; i < drm.res->count_connectors; i++)
 		drmModeFreeConnector(drm.connectors[i]);
 
@@ -1376,8 +1431,9 @@ static void setup_sink_crc(void)
 static void setup_crcs(void)
 {
 	enum pixel_format f;
+	int crtc_idx = kmstest_get_crtc_idx(drm.res, prim_mode_params.crtc_id);
 
-	pipe_crc = igt_pipe_crc_new(0, INTEL_PIPE_CRC_SOURCE_AUTO);
+	pipe_crc = igt_pipe_crc_new(crtc_idx, INTEL_PIPE_CRC_SOURCE_AUTO);
 
 	setup_sink_crc();
 
@@ -1448,10 +1504,23 @@ static bool fbc_supported_on_chipset(void)
 
 static void setup_fbc(void)
 {
+	drmModeConnectorPtr c = get_connector(prim_mode_params.connector_id);
+
 	if (!fbc_supported_on_chipset()) {
 		igt_info("Can't test FBC: not supported on this chipset\n");
 		return;
 	}
+
+	/*
+	 * While some platforms do allow FBC on pipes B/C, this test suite
+	 * is not prepared for that yet.
+	 * TODO: solve this.
+	 */
+	if (!connector_supports_pipe_a(c)) {
+		igt_info("Can't test FBC: primary connector doesn't support "
+			 "pipe A\n");
+		return;
+	}
 	fbc.can_test = true;
 
 	fbc_setup_last_action();
@@ -2212,16 +2281,12 @@ static void wait_flip_event(void)
 
 static void set_prim_plane_for_params(struct modeset_params *params)
 {
-	int rc, i, crtc_index = -1;
+	int rc, i;
+	int crtc_idx = kmstest_get_crtc_idx(drm.res, params->crtc_id);
 	uint32_t plane_id = 0;
 
-	for (i = 0; i < drm.res->count_crtcs; i++)
-		if (drm.res->crtcs[i] == params->crtc_id)
-			crtc_index = i;
-	igt_assert(crtc_index >= 0);
-
 	for (i = 0; i < drm.plane_res->count_planes; i++)
-		if ((drm.planes[i]->possible_crtcs & (1 << crtc_index)) &&
+		if ((drm.planes[i]->possible_crtcs & (1 << crtc_idx)) &&
 		    drm.plane_types[i] == DRM_PLANE_TYPE_PRIMARY)
 			plane_id = drm.planes[i]->plane_id;
 	igt_assert(plane_id);
@@ -2518,10 +2583,10 @@ static bool prim_plane_disabled(void)
 {
 	int i, rc;
 	bool disabled, found = false;
+	int crtc_idx = kmstest_get_crtc_idx(drm.res, prim_mode_params.crtc_id);
 
 	for (i = 0; i < drm.plane_res->count_planes; i++) {
-		/* We just pick the first CRTC for the primary plane. */
-		if ((drm.planes[i]->possible_crtcs & 0x1) &&
+		if ((drm.planes[i]->possible_crtcs & (1 << crtc_idx)) &&
 		    drm.plane_types[i] == DRM_PLANE_TYPE_PRIMARY) {
 			found = true;
 			disabled = (drm.planes[i]->crtc_id == 0);
@@ -2629,6 +2694,8 @@ static void scaledprimary_subtest(const struct test_mode *t)
 	struct modeset_params *params = pick_params(t);
 	int i, rc;
 	uint32_t plane_id;
+	int prim_crtc_idx = kmstest_get_crtc_idx(drm.res,
+						 prim_mode_params.crtc_id);
 
 	igt_require_f(intel_gen(intel_get_drm_devid(drm.fd)) >= 9,
 		      "Can't test primary plane scaling before gen 9\n");
@@ -2658,7 +2725,7 @@ static void scaledprimary_subtest(const struct test_mode *t)
 			 pick_color(&new_fb, COLOR_MAGENTA));
 
 	for (i = 0; i < drm.plane_res->count_planes; i++)
-		if ((drm.planes[i]->possible_crtcs & 1) &&
+		if ((drm.planes[i]->possible_crtcs & (1 << prim_crtc_idx)) &&
 		    drm.plane_types[i] == DRM_PLANE_TYPE_PRIMARY)
 			plane_id = drm.planes[i]->plane_id;
 
@@ -3068,6 +3135,8 @@ static void tilingchange_subtest(const struct test_mode *t)
 		page_flip_for_params(params, flip_type);
 		do_assertions(0);
 	}
+
+	igt_remove_fb(drm.fd, &new_fb);
 }
 
 /*
@@ -3126,6 +3195,8 @@ static void basic_subtest(const struct test_mode *t)
 		update_wanted_crc(t, &pattern->crcs[t->format][r]);
 		do_assertions(assertions);
 	}
+
+	igt_remove_fb(drm.fd, &fb2);
 }
 
 static int opt_handler(int option, int option_index, void *data)
