@@ -226,27 +226,6 @@ static void disable_or_dpms_all_screens(struct mode_set_data *data, bool dpms)
 	igt_assert(wait_for_suspended()); \
 } while (0)
 
-static uint32_t find_crtc_for_connector(drmModeResPtr res,
-					drmModeConnectorPtr connector)
-{
-	drmModeEncoderPtr e;
-	uint32_t crtc_id = 0;
-	int i, j;
-
-	for (i = 0; i < connector->count_encoders && !crtc_id; i++) {
-		e = drmModeGetEncoder(drm_fd, connector->encoders[i]);
-
-		for (j = 0; (e->possible_crtcs >> j) && !crtc_id; j++)
-			if (e->possible_crtcs & (1 << j))
-				crtc_id = res->crtcs[j];
-
-		drmModeFreeEncoder(e);
-	}
-
-	igt_assert(crtc_id);
-	return crtc_id;
-}
-
 static bool init_modeset_params_for_type(struct mode_set_data *data,
 					 struct modeset_params *params,
 					 enum screen_type type)
@@ -280,7 +259,8 @@ static bool init_modeset_params_for_type(struct mode_set_data *data,
 			      DRM_FORMAT_XRGB8888, LOCAL_DRM_FORMAT_MOD_NONE,
 			      &params->fb);
 
-	params->crtc_id = find_crtc_for_connector(data->res, connector);
+	params->crtc_id = kmstest_find_crtc_for_connector(drm_fd, data->res,
+							  connector, 0);
 	params->connector_id = connector->connector_id;
 	params->mode = mode;
 
@@ -1357,15 +1337,13 @@ static void reg_read_ioctl_subtest(void)
 
 static bool device_in_pci_d3(void)
 {
-	struct pci_device *pci_dev;
-	int rc;
 	uint16_t val;
+	int rc;
 
-	pci_dev = intel_get_pci_device();
-
-	rc = pci_device_cfg_read_u16(pci_dev, &val, 0xd4);
+	rc = pci_device_cfg_read_u16(intel_get_pci_device(), &val, 0xd4);
 	igt_assert_eq(rc, 0);
 
+	igt_debug("%s: PCI D3 state=%d\n", __func__, val & 0x3);
 	return (val & 0x3) == 0x3;
 }
 
@@ -1374,11 +1352,9 @@ static void pci_d3_state_subtest(void)
 	igt_require(has_runtime_pm);
 
 	disable_all_screens_and_wait(&ms_data);
-
-	igt_assert(device_in_pci_d3());
+	igt_assert(igt_wait(device_in_pci_d3(), 2000, 100));
 
 	enable_one_screen_and_wait(&ms_data);
-
 	igt_assert(!device_in_pci_d3());
 }
 
@@ -1681,17 +1657,6 @@ static void test_one_plane(bool dpms, uint32_t plane_id,
 	igt_assert(wait_for_suspended());
 }
 
-static int get_crtc_idx(drmModeResPtr res, uint32_t crtc_id)
-{
-	int i;
-
-	for (i = 0; i < res->count_crtcs; i++)
-		if (res->crtcs[i] == crtc_id)
-			return i;
-
-	igt_assert(false);
-}
-
 /* This one also triggered WARNs on our driver at some point in time. */
 static void planes_subtest(bool universal, bool dpms)
 {
@@ -1699,7 +1664,8 @@ static void planes_subtest(bool universal, bool dpms)
 	drmModePlaneResPtr planes;
 
 	igt_require(default_mode_params);
-	crtc_idx = get_crtc_idx(ms_data.res, default_mode_params->crtc_id);
+	crtc_idx = kmstest_get_crtc_idx(ms_data.res,
+					default_mode_params->crtc_id);
 
 	if (universal) {
 		rc = drmSetClientCap(drm_fd, DRM_CLIENT_CAP_UNIVERSAL_PLANES,
@@ -1800,7 +1766,8 @@ static void pm_test_caching(void)
 	uint32_t handle;
 	uint8_t *gem_buf;
 
-	uint32_t i, got_caching;
+	uint32_t i;
+	uint32_t default_cache_level;
 	uint32_t gtt_obj_max_size = (16 * 1024);
 	uint32_t cache_levels[3] = {
 		I915_CACHING_NONE,
@@ -1808,42 +1775,34 @@ static void pm_test_caching(void)
 		I915_CACHING_DISPLAY,           /* eDRAM caching */
 	};
 
+	disable_all_screens(&ms_data);
 
 	handle = gem_create(drm_fd, gtt_obj_max_size);
+	default_cache_level = gem_get_caching(drm_fd, handle);
 	gem_buf = gem_mmap__gtt(drm_fd, handle, gtt_obj_max_size, PROT_WRITE);
 
 	for (i = 0; i < ARRAY_SIZE(cache_levels); i++) {
+		igt_assert(wait_for_suspended());
+		gem_set_caching(drm_fd, handle, default_cache_level);
+
+		/* Ensure we bind the vma into the GGTT */
 		memset(gem_buf, 16 << i, gtt_obj_max_size);
 
-		disable_all_screens_and_wait(&ms_data);
-
-		igt_debug("Setting cache level %u\n", cache_levels[i]);
-
-		gem_set_caching(drm_fd, handle, cache_levels[i]);
-
-		got_caching = gem_get_caching(drm_fd, handle);
-
-		igt_debug("Got back %u\n", got_caching);
-
-		/*
-		 * Allow fall-back to CACHING_NONE in case the platform does
-		 * not support it.
+		/* Now try changing the cache-level on the bound object.
+		 * This will either unlikely unbind the object from the GGTT,
+		 * or more likely just change the PTEs inside the GGTT. Either
+		 * way the driver must take the rpm wakelock around the GSM
+		 * access.
 		 */
-		if (cache_levels[i] == I915_CACHING_DISPLAY)
-			igt_assert(got_caching == I915_CACHING_NONE ||
-				   got_caching == I915_CACHING_DISPLAY);
-		else
-			igt_assert(got_caching == cache_levels[i]);
-
-		enable_one_screen_and_wait(&ms_data);
+		igt_debug("Setting cache level %u\n", cache_levels[i]);
+		igt_assert(wait_for_suspended());
+		gem_set_caching(drm_fd, handle, cache_levels[i]);
 	}
 
 	igt_assert(munmap(gem_buf, gtt_obj_max_size) == 0);
 	gem_close(drm_fd, handle);
 }
 
-
-
 static void fences_subtest(bool dpms)
 {
 	int i;
