@@ -27,6 +27,7 @@
 
 #include <errno.h>
 #include <fcntl.h>
+#include <getopt.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -40,8 +41,6 @@
 #include "intel_chipset.h"
 #include "drmtest.h"
 
-static uint32_t devid = -1;
-
 /* no bother to include "edid.h" */
 #define _H_ACTIVE(x) (x[2] + ((x[4] & 0xF0) << 4))
 #define _H_SYNC_OFF(x) (x[8] + ((x[11] & 0xC0) << 2))
@@ -53,16 +52,6 @@ static uint32_t devid = -1;
 #define _V_BLANK(x) (x[6] + ((x[7] & 0x0F) << 8))
 #define _PIXEL_CLOCK(x) (x[0] + (x[1] << 8)) * 10000
 
-uint8_t *VBIOS;
-
-#define INTEL_BIOS_8(_addr)	(VBIOS[_addr])
-#define INTEL_BIOS_16(_addr)	(VBIOS[_addr] | \
-				 (VBIOS[_addr + 1] << 8))
-#define INTEL_BIOS_32(_addr)	(VBIOS[_addr] | \
-				 (VBIOS[_addr + 1] << 8) | \
-				 (VBIOS[_addr + 2] << 16) | \
-				 (VBIOS[_addr + 3] << 24))
-
 #define YESNO(val) ((val) ? "yes" : "no")
 
 /* This is not for mapping to memory layout. */
@@ -72,10 +61,16 @@ struct bdb_block {
 	const void *data;
 };
 
-const struct bdb_lvds_lfp_data_ptrs *lvds_lfp_data_ptrs;
-static int tv_present;
-static int lvds_present;
-static int panel_type;
+struct context {
+	const struct vbt_header *vbt;
+	const struct bdb_header *bdb;
+	int size;
+
+	uint32_t devid;
+	int panel_type;
+	bool dump_all_panel_types;
+	bool hexdump;
+};
 
 /* Get BDB block size given a pointer to Block ID. */
 static uint32_t _get_blocksize(const uint8_t *block_base)
@@ -87,9 +82,10 @@ static uint32_t _get_blocksize(const uint8_t *block_base)
 		return *((const uint16_t *)(block_base + 1));
 }
 
-static struct bdb_block *find_section(const struct bdb_header *bdb,
-				      int section_id, int length)
+static struct bdb_block *find_section(struct context *context, int section_id)
 {
+	const struct bdb_header *bdb = context->bdb;
+	int length = context->size;
 	struct bdb_block *block;
 	const uint8_t *base = (const uint8_t *)bdb;
 	int index = 0;
@@ -105,7 +101,7 @@ static struct bdb_block *find_section(const struct bdb_header *bdb,
 	block = malloc(sizeof(*block));
 	if (!block) {
 		fprintf(stderr, "out of memory\n");
-		exit(-1);
+		exit(EXIT_FAILURE);
 	}
 
 	/* walk the sections looking for section_id */
@@ -131,7 +127,7 @@ static struct bdb_block *find_section(const struct bdb_header *bdb,
 	return NULL;
 }
 
-static void dump_general_features(const struct bdb_header *bdb,
+static void dump_general_features(struct context *context,
 				  const struct bdb_block *block)
 {
 	const struct bdb_general_features *features = block->data;
@@ -158,10 +154,13 @@ static void dump_general_features(const struct bdb_header *bdb,
 	printf("\tExternal VBT: %s\n", YESNO(features->download_ext_vbt));
 	printf("\tEnable SSC: %s\n", YESNO(features->enable_ssc));
 	if (features->enable_ssc) {
-		if (IS_VALLEYVIEW(devid) || IS_CHERRYVIEW(devid) ||
-		    IS_BROXTON(devid))
+		if (!context->devid)
+			printf("\tSSC frequency: <unknown platform>\n");
+		else if (IS_VALLEYVIEW(context->devid) ||
+			 IS_CHERRYVIEW(context->devid) ||
+			 IS_BROXTON(context->devid))
 			printf("\tSSC frequency: 100 MHz\n");
-		else if (HAS_PCH_SPLIT(devid))
+		else if (HAS_PCH_SPLIT(context->devid))
 			printf("\tSSC frequency: %s\n", features->ssc_freq ?
 			       "100 MHz" : "120 MHz");
 		else
@@ -179,12 +178,9 @@ static void dump_general_features(const struct bdb_header *bdb,
 	       YESNO(features->legacy_monitor_detect));
 	printf("\tIntegrated CRT: %s\n", YESNO(features->int_crt_support));
 	printf("\tIntegrated TV: %s\n", YESNO(features->int_tv_support));
-
-	tv_present = 1;		/* should be based on whether TV DAC exists */
-	lvds_present = 1;	/* should be based on IS_MOBILE() */
 }
 
-static void dump_backlight_info(const struct bdb_header *bdb,
+static void dump_backlight_info(struct context *context,
 				const struct bdb_block *block)
 {
 	const struct bdb_lvds_backlight *backlight = block->data;
@@ -196,7 +192,7 @@ static void dump_backlight_info(const struct bdb_header *bdb,
 		return;
 	}
 
-	blc = &backlight->panels[panel_type];
+	blc = &backlight->panels[context->panel_type];
 
 	printf("\tInverter type: %d\n", blc->inverter_type);
 	printf("\t     polarity: %d\n", blc->inverter_polarity);
@@ -367,7 +363,7 @@ static const char *efp_conn(uint8_t type)
 
 
 
-static void dump_child_device(const struct bdb_header *bdb,
+static void dump_child_device(struct context *context,
 			      struct child_device_config *child)
 {
 	char child_id[11];
@@ -375,7 +371,7 @@ static void dump_child_device(const struct bdb_header *bdb,
 	if (!child->device_type)
 		return;
 
-	if (bdb->version < 152) {
+	if (context->bdb->version < 152) {
 		strncpy(child_id, (char *)child->device_id, 10);
 		child_id[10] = 0;
 
@@ -406,7 +402,7 @@ static void dump_child_device(const struct bdb_header *bdb,
 	}
 }
 
-static void dump_general_definitions(const struct bdb_header *bdb,
+static void dump_general_definitions(struct context *context,
 				     const struct bdb_block *block)
 {
 	const struct bdb_general_definitions *defs = block->data;
@@ -421,15 +417,14 @@ static void dump_general_definitions(const struct bdb_header *bdb,
 	printf("\tUse DPMS on AIM devices: %s\n", YESNO(defs->dpms_aim));
 	printf("\tBoot display type: 0x%02x%02x\n", defs->boot_display[1],
 	       defs->boot_display[0]);
-	printf("\tTV data block present: %s\n", YESNO(tv_present));
 	printf("\tChild device size: %d\n", defs->child_dev_size);
 	child_device_num = (block->size - sizeof(*defs)) /
 		defs->child_dev_size;
 	for (i = 0; i < child_device_num; i++)
-		dump_child_device(bdb,(void*)&defs->devices[i * defs->child_dev_size]);
+		dump_child_device(context, (void*)&defs->devices[i * defs->child_dev_size]);
 }
 
-static void dump_child_devices(const struct bdb_header *bdb,
+static void dump_child_devices(struct context *context,
 			       const struct bdb_block *block)
 {
 	const struct bdb_child_devices *child_devs = block->data;
@@ -453,13 +448,16 @@ static void dump_child_devices(const struct bdb_header *bdb,
 	}
 }
 
-static void dump_lvds_options(const struct bdb_header *bdb,
+static void dump_lvds_options(struct context *context,
 			      const struct bdb_block *block)
 {
 	const struct bdb_lvds_options *options = block->data;
 
-	panel_type = options->panel_type;
-	printf("\tPanel type: %d\n", panel_type);
+	if (context->panel_type == options->panel_type)
+		printf("\tPanel type: %d\n", options->panel_type);
+	else
+		printf("\tPanel type: %d (override %d)\n",
+		       options->panel_type, context->panel_type);
 	printf("\tLVDS EDID available: %s\n", YESNO(options->lvds_edid));
 	printf("\tPixel dither: %s\n", YESNO(options->pixel_dither));
 	printf("\tPFIT auto ratio: %s\n", YESNO(options->pfit_ratio_auto));
@@ -470,22 +468,20 @@ static void dump_lvds_options(const struct bdb_header *bdb,
 	printf("\tPFIT mode: %d\n", options->pfit_mode);
 }
 
-static void dump_lvds_ptr_data(const struct bdb_header *bdb,
+static void dump_lvds_ptr_data(struct context *context,
 			       const struct bdb_block *block)
 {
 	const struct bdb_lvds_lfp_data_ptrs *ptrs = block->data;
 
 	printf("\tNumber of entries: %d\n", ptrs->lvds_entries);
-
-	/* save for use by dump_lvds_data() */
-	lvds_lfp_data_ptrs = ptrs;
 }
 
-static void dump_lvds_data(const struct bdb_header *bdb,
+static void dump_lvds_data(struct context *context,
 			   const struct bdb_block *block)
 {
 	const struct bdb_lvds_lfp_data *lvds_data = block->data;
-	const struct bdb_lvds_lfp_data_ptrs *ptrs = lvds_lfp_data_ptrs;
+	struct bdb_block *ptrs_block;
+	const struct bdb_lvds_lfp_data_ptrs *ptrs;
 	int num_entries;
 	int i;
 	int hdisplay, hsyncstart, hsyncend, htotal;
@@ -493,11 +489,14 @@ static void dump_lvds_data(const struct bdb_header *bdb,
 	float clock;
 	int lfp_data_size, dvo_offset;
 
-	if (!ptrs) {
+	ptrs_block = find_section(context, BDB_LVDS_LFP_DATA_PTRS);
+	if (!ptrs_block) {
 		printf("No LVDS ptr block\n");
 		return;
 	}
 
+	ptrs = ptrs_block->data;
+
 	lfp_data_size =
 	    ptrs->ptr[1].fp_timing_offset - ptrs->ptr[0].fp_timing_offset;
 	dvo_offset =
@@ -516,7 +515,10 @@ static void dump_lvds_data(const struct bdb_header *bdb,
 		    (const struct bdb_lvds_lfp_data_entry *)lfp_data_ptr;
 		char marker;
 
-		if (i == panel_type)
+		if (i != context->panel_type && !context->dump_all_panel_types)
+			continue;
+
+		if (i == context->panel_type)
 			marker = '*';
 		else
 			marker = ' ';
@@ -552,9 +554,11 @@ static void dump_lvds_data(const struct bdb_header *bdb,
 		       (hsyncend > htotal || vsyncend > vtotal) ?
 		       "BAD!" : "good");
 	}
+
+	free(ptrs_block);
 }
 
-static void dump_driver_feature(const struct bdb_header *bdb,
+static void dump_driver_feature(struct context *context,
 				const struct bdb_block *block)
 {
 	const struct bdb_driver_feature *feature = block->data;
@@ -621,7 +625,7 @@ static void dump_driver_feature(const struct bdb_header *bdb,
 	       feature->legacy_crt_max_refresh);
 }
 
-static void dump_edp(const struct bdb_header *bdb,
+static void dump_edp(struct context *context,
 		     const struct bdb_block *block)
 {
 	const struct bdb_edp *edp = block->data;
@@ -629,7 +633,10 @@ static void dump_edp(const struct bdb_header *bdb,
 	int i;
 
 	for (i = 0; i < 16; i++) {
-		printf("\tPanel %d%s\n", i, panel_type == i ? " *" : "");
+		if (i != context->panel_type && !context->dump_all_panel_types)
+			continue;
+
+		printf("\tPanel %d%s\n", i, context->panel_type == i ? " *" : "");
 
 		printf("\t\tPower Sequence: T3 %d T7 %d T9 %d T10 %d T12 %d\n",
 		       edp->power_seqs[i].t3,
@@ -751,7 +758,7 @@ print_detail_timing_data(const struct lvds_dvo_timing2 *dvo_timing)
 	printf("\tclock: %d\n", dvo_timing->clock * 10);
 }
 
-static void dump_sdvo_panel_dtds(const struct bdb_header *bdb,
+static void dump_sdvo_panel_dtds(struct context *context,
 				 const struct bdb_block *block)
 {
 	const struct lvds_dvo_timing2 *dvo_timing = block->data;
@@ -764,7 +771,7 @@ static void dump_sdvo_panel_dtds(const struct bdb_header *bdb,
 	}
 }
 
-static void dump_sdvo_lvds_options(const struct bdb_header *bdb,
+static void dump_sdvo_lvds_options(struct context *context,
 				   const struct bdb_block *block)
 {
 	const struct bdb_sdvo_lvds_options *options = block->data;
@@ -786,15 +793,15 @@ static void dump_sdvo_lvds_options(const struct bdb_header *bdb,
 	printf("\tmisc[3]: %x\n", options->panel_misc_bits_4);
 }
 
-static void dump_mipi_config(const struct bdb_header *bdb,
+static void dump_mipi_config(struct context *context,
 			     const struct bdb_block *block)
 {
 	const struct bdb_mipi_config *start = block->data;
 	const struct mipi_config *config;
 	const struct mipi_pps_data *pps;
 
-	config = &start->config[panel_type];
-	pps = &start->pps[panel_type];
+	config = &start->config[context->panel_type];
+	pps = &start->pps[context->panel_type];
 
 	printf("\tGeneral Param\n");
 	printf("\t\t BTA disable: %s\n", config->bta ? "Disabled" : "Enabled");
@@ -1173,7 +1180,7 @@ static int goto_next_sequence_v3(const uint8_t *data, int index, int total)
 	return 0;
 }
 
-static void dump_mipi_sequence(const struct bdb_header *bdb,
+static void dump_mipi_sequence(struct context *context,
 			       const struct bdb_block *block)
 {
 	const struct bdb_mipi_sequence *sequence = block->data;
@@ -1197,7 +1204,7 @@ static void dump_mipi_sequence(const struct bdb_header *bdb,
 		return;
 	}
 
-	data = find_panel_sequence_block(sequence, panel_type,
+	data = find_panel_sequence_block(sequence, context->panel_type,
 					 block->size, &seq_size);
 	if (!data)
 		return;
@@ -1231,6 +1238,25 @@ static void dump_mipi_sequence(const struct bdb_header *bdb,
 			dump_sequence(sequence_ptrs[i], sequence->version);
 }
 
+/* get panel type from lvds options block, or -1 if block not found */
+static int get_panel_type(struct context *context)
+{
+	struct bdb_block *block;
+	const struct bdb_lvds_options *options;
+	int panel_type;
+
+	block = find_section(context, BDB_LVDS_OPTIONS);
+	if (!block)
+		return -1;
+
+	options = block->data;
+	panel_type = options->panel_type;
+
+	free(block);
+
+	return panel_type;
+}
+
 static int
 get_device_id(unsigned char *bios, int size)
 {
@@ -1254,7 +1280,7 @@ get_device_id(unsigned char *bios, int size)
 struct dumper {
 	uint8_t id;
 	const char *name;
-	void (*dump)(const struct bdb_header *bdb,
+	void (*dump)(struct context *context,
 		     const struct bdb_block *block);
 };
 
@@ -1347,20 +1373,15 @@ static void hex_dump(const struct bdb_block *block)
 	printf("\n\n");
 }
 
-static void dump_section(const struct bdb_header *bdb, int section_id, int size)
+static bool dump_section(struct context *context, int section_id)
 {
 	struct dumper *dumper = NULL;
-	const struct bdb_block *block;
-	static int done[256];
+	struct bdb_block *block;
 	int i;
 
-	if (done[section_id])
-		return;
-	done[section_id] = 1;
-
-	block = find_section(bdb, section_id, size);
+	block = find_section(context, section_id);
 	if (!block)
-		return;
+		return false;
 
 	for (i = 0; i < ARRAY_SIZE(dumpers); i++) {
 		if (block->id == dumpers[i].id) {
@@ -1374,45 +1395,171 @@ static void dump_section(const struct bdb_header *bdb, int section_id, int size)
 	else
 		printf("BDB block %d:\n", block->id);
 
-	hex_dump(block);
+	if (context->hexdump)
+		hex_dump(block);
 	if (dumper && dumper->dump)
-		dumper->dump(bdb, block);
+		dumper->dump(context, block);
 	printf("\n");
+
+	free(block);
+
+	return true;
+}
+
+static void dump_headers(struct context *context)
+{
+	const struct vbt_header *vbt = context->vbt;
+	const struct bdb_header *bdb = context->bdb;
+	int i, j = 0;
+
+	printf("VBT signature:\t\"%.*s\"\n",
+	       (int)sizeof(vbt->signature), vbt->signature);
+	printf("VBT version:\t%d.%d\n", vbt->version / 100, vbt->version % 100);
+
+	printf("BDB signature:\t\"%.*s\"\n",
+	       (int)sizeof(bdb->signature), bdb->signature);
+	printf("BDB version:\t%d\n", bdb->version);
+
+	printf("BDB blocks present:");
+	for (i = 0; i < 256; i++) {
+		struct bdb_block *block;
+
+		block = find_section(context, i);
+		if (!block)
+			continue;
+
+		if (j++ % 16)
+			printf(" %3d", i);
+		else
+			printf("\n\t%3d", i);
+
+		free(block);
+	}
+	printf("\n\n");
+}
+
+enum opt {
+	OPT_UNKNOWN = '?',
+	OPT_END = -1,
+	OPT_FILE,
+	OPT_DEVID,
+	OPT_PANEL_TYPE,
+	OPT_ALL_PANELS,
+	OPT_HEXDUMP,
+	OPT_BLOCK,
+	OPT_USAGE,
+};
+
+static void usage(const char *toolname)
+{
+	fprintf(stderr, "usage: %s", toolname);
+	fprintf(stderr, " --file=<rom_file>"
+			" [--devid=<device_id>]"
+			" [--panel-type=<panel_type>]"
+			" [--all-panels]"
+			" [--hexdump]"
+			" [--block=<block_no>]"
+			" [--help]\n");
 }
 
 int main(int argc, char **argv)
 {
+	uint8_t *VBIOS;
+	int index;
+	enum opt opt;
 	int fd;
 	struct vbt_header *vbt = NULL;
 	int vbt_off, bdb_off, i;
-	const char *filename = "bios";
+	const char *filename = NULL;
+	const char *toolname = argv[0];
 	struct stat finfo;
 	int size;
-	struct bdb_block *block;
-	struct bdb_header *bdb;
-	char signature[17];
-	char *devid_string;
+	struct context context = {
+		.panel_type = -1,
+	};
+	char *endp;
+	int block_number = -1;
 
-	if (argc != 2) {
-		printf("usage: %s <rom file>\n", argv[0]);
-		return 1;
+	static struct option options[] = {
+		{ "file",	required_argument,	NULL,	OPT_FILE },
+		{ "devid",	required_argument,	NULL,	OPT_DEVID },
+		{ "panel-type",	required_argument,	NULL,	OPT_PANEL_TYPE },
+		{ "all-panels",	no_argument,		NULL,	OPT_ALL_PANELS },
+		{ "hexdump",	no_argument,		NULL,	OPT_HEXDUMP },
+		{ "block",	required_argument,	NULL,	OPT_BLOCK },
+		{ "help",	no_argument,		NULL,	OPT_USAGE },
+		{ 0 }
+	};
+
+	for (opt = 0; opt != OPT_END; ) {
+		opt = getopt_long(argc, argv, "", options, &index);
+
+		switch (opt) {
+		case OPT_FILE:
+			filename = optarg;
+			break;
+		case OPT_DEVID:
+			context.devid = strtoul(optarg, &endp, 16);
+			if (!context.devid || *endp) {
+				fprintf(stderr, "invalid devid '%s'\n", optarg);
+				return EXIT_FAILURE;
+			}
+			break;
+		case OPT_PANEL_TYPE:
+			context.panel_type = strtoul(optarg, &endp, 0);
+			if (*endp || context.panel_type > 15) {
+				fprintf(stderr, "invalid panel type '%s'\n",
+					optarg);
+				return EXIT_FAILURE;
+			}
+			break;
+		case OPT_ALL_PANELS:
+			context.dump_all_panel_types = true;
+			break;
+		case OPT_HEXDUMP:
+			context.hexdump = true;
+			break;
+		case OPT_BLOCK:
+			block_number = strtoul(optarg, &endp, 0);
+			if (*endp) {
+				fprintf(stderr, "invalid block number '%s'\n",
+					optarg);
+				return EXIT_FAILURE;
+			}
+			break;
+		case OPT_END:
+			break;
+		case OPT_USAGE: /* fall-through */
+		case OPT_UNKNOWN:
+			usage(toolname);
+			return EXIT_FAILURE;
+		}
 	}
 
-	if ((devid_string = getenv("DEVICE")))
-	    devid = strtoul(devid_string, NULL, 0);
+	argc -= optind;
+	argv += optind;
 
-	filename = argv[1];
+	if (!filename) {
+		if (argc == 1) {
+			/* for backwards compatibility */
+			filename = argv[0];
+		} else {
+			usage(toolname);
+			return EXIT_FAILURE;
+		}
+	}
 
 	fd = open(filename, O_RDONLY);
 	if (fd == -1) {
-		printf("Couldn't open \"%s\": %s\n", filename, strerror(errno));
-		return 1;
+		fprintf(stderr, "Couldn't open \"%s\": %s\n",
+			filename, strerror(errno));
+		return EXIT_FAILURE;
 	}
 
 	if (stat(filename, &finfo)) {
-		printf("failed to stat \"%s\": %s\n", filename,
-		       strerror(errno));
-		return 1;
+		fprintf(stderr, "Failed to stat \"%s\": %s\n",
+			filename, strerror(errno));
+		return EXIT_FAILURE;
 	}
 	size = finfo.st_size;
 
@@ -1422,9 +1569,9 @@ int main(int argc, char **argv)
 		VBIOS = malloc (size);
 		while ((ret = read(fd, VBIOS + len, size - len))) {
 			if (ret < 0) {
-				printf("failed to read \"%s\": %s\n", filename,
-				       strerror(errno));
-				return 1;
+				fprintf(stderr, "Failed to read \"%s\": %s\n",
+					filename, strerror(errno));
+				return EXIT_FAILURE;
 			}
 
 			len += ret;
@@ -1436,8 +1583,9 @@ int main(int argc, char **argv)
 	} else {
 		VBIOS = mmap(NULL, size, PROT_READ, MAP_SHARED, fd, 0);
 		if (VBIOS == MAP_FAILED) {
-			printf("failed to map \"%s\": %s\n", filename, strerror(errno));
-			return 1;
+			fprintf(stderr, "Failed to map \"%s\": %s\n",
+				filename, strerror(errno));
+			return EXIT_FAILURE;
 		}
 	}
 
@@ -1451,58 +1599,50 @@ int main(int argc, char **argv)
 	}
 
 	if (!vbt) {
-		printf("VBT signature missing\n");
-		return 1;
+		fprintf(stderr, "VBT signature missing\n");
+		return EXIT_FAILURE;
 	}
 
-	printf("VBT vers: %d.%d\n", vbt->version / 100, vbt->version % 100);
-
 	bdb_off = vbt_off + vbt->bdb_offset;
 	if (bdb_off >= size - sizeof(struct bdb_header)) {
-		printf("Invalid VBT found, BDB points beyond end of data block\n");
-		return 1;
+		fprintf(stderr, "Invalid VBT found, BDB points beyond end of data block\n");
+		return EXIT_FAILURE;
 	}
 
-	bdb = (struct bdb_header *)(VBIOS + bdb_off);
-	strncpy(signature, (char *)bdb->signature, 16);
-	signature[16] = 0;
-	printf("BDB sig: %s\n", signature);
-	printf("BDB vers: %d\n", bdb->version);
+	context.vbt = vbt;
+	context.bdb = (const struct bdb_header *)(VBIOS + bdb_off);
+	context.size = size;
 
-	printf("Available sections: ");
-
-	for (i = 0; i < 256; i++) {
-		block = find_section(bdb, i, size);
-		if (!block)
-			continue;
-		printf("%d ", i);
-		free(block);
+	if (!context.devid) {
+		const char *devid_string = getenv("DEVICE");
+		if (devid_string)
+			context.devid = strtoul(devid_string, NULL, 16);
 	}
-	printf("\n");
+	if (!context.devid)
+		context.devid = get_device_id(VBIOS, size);
+	if (!context.devid)
+		fprintf(stderr, "Warning: could not find PCI device ID!\n");
 
-	if (devid == -1)
-	    devid = get_device_id(VBIOS, size);
-	if (devid == -1)
-	    printf("Warning: could not find PCI device ID!\n");
+	if (context.panel_type == -1)
+		context.panel_type = get_panel_type(&context);
+	if (context.panel_type == -1) {
+		fprintf(stderr, "Warning: panel type not set, using 0\n");
+		context.panel_type = 0;
+	}
 
-	dump_section(bdb, BDB_GENERAL_FEATURES, size);
-	dump_section(bdb, BDB_GENERAL_DEFINITIONS, size);
-	dump_section(bdb, BDB_CHILD_DEVICE_TABLE, size);
-	dump_section(bdb, BDB_LVDS_OPTIONS, size);
-	dump_section(bdb, BDB_LVDS_LFP_DATA_PTRS, size);
-	dump_section(bdb, BDB_LVDS_LFP_DATA, size);
-	dump_section(bdb, BDB_LVDS_BACKLIGHT, size);
+	if (block_number != -1) {
+		/* dump specific section only */
+		if (!dump_section(&context, block_number)) {
+			fprintf(stderr, "Block %d not found\n", block_number);
+			return EXIT_FAILURE;
+		}
+	} else {
+		dump_headers(&context);
 
-	dump_section(bdb, BDB_SDVO_LVDS_OPTIONS, size);
-	dump_section(bdb, BDB_SDVO_PANEL_DTDS, size);
-
-	dump_section(bdb, BDB_DRIVER_FEATURES, size);
-	dump_section(bdb, BDB_EDP, size);
-	dump_section(bdb, BDB_MIPI_CONFIG, size);
-	dump_section(bdb, BDB_MIPI_SEQUENCE, size);
-
-	for (i = 0; i < 256; i++)
-		dump_section(bdb, i, size);
+		/* dump all sections  */
+		for (i = 0; i < 256; i++)
+			dump_section(&context, i);
+	}
 
 	return 0;
 }
